@@ -37,7 +37,7 @@ from pyrogram.api.errors import (
     PhoneNumberUnoccupied, PhoneCodeInvalid, PhoneCodeHashEmpty,
     PhoneCodeExpired, PhoneCodeEmpty, SessionPasswordNeeded,
     PasswordHashInvalid, FloodWait, PeerIdInvalid, FilePartMissing,
-    ChatAdminRequired
+    ChatAdminRequired, FirstnameInvalid
 )
 from pyrogram.api.types import (
     User, Chat, Channel,
@@ -47,8 +47,8 @@ from pyrogram.api.types import (
     InputPeerUser, InputPeerChat, InputPeerChannel
 )
 from pyrogram.crypto import CTR
-from pyrogram.extensions import Markdown
 from pyrogram.session import Auth, Session
+from .style import Markdown, HTML
 
 log = logging.getLogger(__name__)
 
@@ -64,21 +64,63 @@ class Client:
     Args:
         session_name (:obj:`str`):
             Name to uniquely identify an authorized session. It will be used
-            to save the session to a file named ``<session_name>.session``.
+            to save the session to a file named *<session_name>.session* and to load
+            it when you restart your script. As long as a valid session file exists,
+            Pyrogram won't ask you again to input your phone number.
 
         test_mode (:obj:`bool`, optional):
             Enable or disable log-in to testing servers. Defaults to False.
             Only applicable for new sessions and will be ignored in case previously
             created sessions are loaded.
+
+        phone_number (:obj:`str`, optional):
+            Pass your phone number (with your Country Code prefix included) to avoid
+            entering it manually. Only applicable for new sessions.
+
+        phone_code (:obj:`str` | :obj:`callable`, optional):
+            Pass the phone code as string (for test numbers only), or pass a callback function
+            which must return the correct phone code as string (e.g., "12345").
+            Only applicable for new sessions.
+
+        password (:obj:`str`, optional):
+            Pass your Two-Step Verification password (if you have one) to avoid entering it
+            manually. Only applicable for new sessions.
+
+        first_name (:obj:`str`, optional):
+            Pass a First Name to avoid entering it manually. It will be used to automatically
+            create a new Telegram account in case the phone number you passed is not registered yet.
+
+        last_name (:obj:`str`, optional):
+            Same purpose as *first_name*; pass a Last Name to avoid entering it manually. It can
+            be an empty string: ""
+
+        workers (:obj:`int`, optional):
+            Thread pool size for handling incoming messages (updates). Defaults to 4.
     """
 
     INVITE_LINK_RE = re.compile(r"^(?:https?://)?t\.me/joinchat/(.+)$")
     DIALOGS_AT_ONCE = 100
 
-    def __init__(self, session_name: str, test_mode: bool = False):
+    def __init__(self,
+                 session_name: str,
+                 test_mode: bool = False,
+                 phone_number: str = None,
+                 phone_code: str or callable = None,
+                 password: str = None,
+                 first_name: str = None,
+                 last_name: str = None,
+                 workers: int = 4):
         self.session_name = session_name
         self.test_mode = test_mode
 
+        self.phone_number = phone_number
+        self.password = password
+        self.phone_code = phone_code
+        self.first_name = first_name
+        self.last_name = last_name
+
+        self.workers = workers
+
         self.dc_id = None
         self.auth_key = None
         self.user_id = None
@@ -89,6 +131,7 @@ class Client:
         self.peers_by_username = {}
 
         self.markdown = Markdown(self.peers_by_id)
+        self.html = HTML(self.peers_by_id)
 
         self.config = None
         self.proxy = None
@@ -99,11 +142,22 @@ class Client:
 
     def start(self):
         """Use this method to start the Client after creating it.
-        Requires no parameters."""
+        Requires no parameters.
+
+        Raises:
+            :class:`pyrogram.Error`
+        """
         self.load_config()
         self.load_session(self.session_name)
 
-        self.session = Session(self.dc_id, self.test_mode, self.proxy, self.auth_key, self.config.api_id)
+        self.session = Session(
+            self.dc_id,
+            self.test_mode,
+            self.proxy,
+            self.auth_key,
+            self.config.api_id,
+            workers=self.workers
+        )
 
         terms = self.session.start()
 
@@ -111,6 +165,7 @@ class Client:
             print("\n".join(terms.splitlines()), "\n")
 
             self.user_id = self.authorize()
+            self.password = None
             self.save_session()
 
         self.rnd_id = self.session.msg_id
@@ -170,21 +225,27 @@ class Client:
         return self.session.send(data)
 
     def authorize(self):
+        phone_number_invalid_raises = self.phone_number is not None
+        phone_code_invalid_raises = self.phone_code is not None
+        password_hash_invalid_raises = self.password is not None
+        first_name_invalid_raises = self.first_name is not None
+
         while True:
-            phone_number = input("Enter phone number: ")
+            if self.phone_number is None:
+                self.phone_number = input("Enter phone number: ")
 
-            while True:
-                confirm = input("Is \"{}\" correct? (y/n): ".format(phone_number))
+                while True:
+                    confirm = input("Is \"{}\" correct? (y/n): ".format(self.phone_number))
 
-                if confirm in ("y", "1"):
-                    break
-                elif confirm in ("n", "2"):
-                    phone_number = input("Enter phone number: ")
+                    if confirm in ("y", "1"):
+                        break
+                    elif confirm in ("n", "2"):
+                        self.phone_number = input("Enter phone number: ")
 
             try:
                 r = self.send(
                     functions.auth.SendCode(
-                        phone_number,
+                        self.phone_number,
                         self.config.api_id,
                         self.config.api_hash
                     )
@@ -195,19 +256,30 @@ class Client:
                 self.dc_id = e.x
                 self.auth_key = Auth(self.dc_id, self.test_mode, self.proxy).create()
 
-                self.session = Session(self.dc_id, self.test_mode, self.proxy, self.auth_key, self.config.api_id)
+                self.session = Session(
+                    self.dc_id,
+                    self.test_mode,
+                    self.proxy,
+                    self.auth_key,
+                    self.config.api_id,
+                    workers=self.workers
+                )
                 self.session.start()
 
                 r = self.send(
                     functions.auth.SendCode(
-                        phone_number,
+                        self.phone_number,
                         self.config.api_id,
                         self.config.api_hash
                     )
                 )
                 break
             except PhoneNumberInvalid as e:
-                print(e.MESSAGE)
+                if phone_number_invalid_raises:
+                    raise
+                else:
+                    print(e.MESSAGE)
+                    self.phone_number = None
             except FloodWait as e:
                 print(e.MESSAGE.format(x=e.x))
                 time.sleep(e.x)
@@ -220,59 +292,80 @@ class Client:
         phone_code_hash = r.phone_code_hash
 
         while True:
-            phone_code = input("Enter phone code: ")
+            self.phone_code = (
+                input("Enter phone code: ") if self.phone_code is None
+                else self.phone_code if type(self.phone_code) is str
+                else self.phone_code()
+            )
 
             try:
                 if phone_registered:
                     r = self.send(
                         functions.auth.SignIn(
-                            phone_number,
+                            self.phone_number,
                             phone_code_hash,
-                            phone_code
+                            self.phone_code
                         )
                     )
                 else:
                     try:
                         self.send(
                             functions.auth.SignIn(
-                                phone_number,
+                                self.phone_number,
                                 phone_code_hash,
-                                phone_code
+                                self.phone_code
                             )
                         )
                     except PhoneNumberUnoccupied:
                         pass
 
-                    first_name = input("First name: ")
-                    last_name = input("Last name: ")
+                    self.first_name = self.first_name if self.first_name is not None else input("First name: ")
+                    self.last_name = self.last_name if self.last_name is not None else input("Last name: ")
 
                     r = self.send(
                         functions.auth.SignUp(
-                            phone_number,
+                            self.phone_number,
                             phone_code_hash,
-                            phone_code,
-                            first_name,
-                            last_name
+                            self.phone_code,
+                            self.first_name,
+                            self.last_name
                         )
                     )
             except (PhoneCodeInvalid, PhoneCodeEmpty, PhoneCodeExpired, PhoneCodeHashEmpty) as e:
-                print(e.MESSAGE)
+                if phone_code_invalid_raises:
+                    raise
+                else:
+                    print(e.MESSAGE)
+                    self.phone_code = None
+            except FirstnameInvalid as e:
+                if first_name_invalid_raises:
+                    raise
+                else:
+                    print(e.MESSAGE)
+                    self.first_name = None
             except SessionPasswordNeeded as e:
                 print(e.MESSAGE)
+                r = self.send(functions.account.GetPassword())
 
                 while True:
                     try:
-                        r = self.send(functions.account.GetPassword())
 
-                        print("Hint: {}".format(r.hint))
-                        password = input("Enter password: ")  # TODO: Use getpass
+                        if self.password is None:
+                            print("Hint: {}".format(r.hint))
+                            self.password = input("Enter password: ")  # TODO: Use getpass
 
-                        password = r.current_salt + password.encode() + r.current_salt
-                        password_hash = sha256(password).digest()
+                        if type(self.password) is str:
+                            self.password = r.current_salt + self.password.encode() + r.current_salt
+
+                        password_hash = sha256(self.password).digest()
 
                         r = self.send(functions.auth.CheckPassword(password_hash))
                     except PasswordHashInvalid as e:
-                        print(e.MESSAGE)
+                        if password_hash_invalid_raises:
+                            raise
+                        else:
+                            print(e.MESSAGE)
+                            self.password = None
                     except FloodWait as e:
                         print(e.MESSAGE.format(x=e.x))
                         time.sleep(e.x)
@@ -431,6 +524,7 @@ class Client:
                     peer_id,
                     peer_access_hash
                 )
+                peer_id = int("-100" + str(peer_id))
             else:
                 continue
 
@@ -440,18 +534,54 @@ class Client:
                 peer_username = peer_username.lower()
                 self.peers_by_username[peer_username] = input_peer
 
-    def resolve_peer(self, chat_id: int or str):
-        if chat_id in ("self", "me"):
+    def resolve_username(self, username: str):
+        username = username.lower().strip("@")
+
+        resolved_peer = self.send(
+            functions.contacts.ResolveUsername(
+                username=username
+            )
+        )  # type: types.contacts.ResolvedPeer
+
+        if type(resolved_peer.peer) is PeerUser:
+            input_peer = InputPeerUser(
+                user_id=resolved_peer.users[0].id,
+                access_hash=resolved_peer.users[0].access_hash
+            )
+            peer_id = input_peer.user_id
+        elif type(resolved_peer.peer) is PeerChannel:
+            input_peer = InputPeerChannel(
+                channel_id=resolved_peer.chats[0].id,
+                access_hash=resolved_peer.chats[0].access_hash
+            )
+            peer_id = int("-100" + str(input_peer.channel_id))
+        else:
+            raise PeerIdInvalid
+
+        self.peers_by_username[username] = input_peer
+        self.peers_by_id[peer_id] = input_peer
+
+        return input_peer
+
+    def resolve_peer(self, peer_id: int or str):
+        if peer_id in ("self", "me"):
             return InputPeerSelf()
         else:
-            try:
-                return (
-                    self.peers_by_username[chat_id.lower().strip("@")]
-                    if isinstance(chat_id, str)
-                    else self.peers_by_id[chat_id]
-                )
-            except KeyError:
-                raise PeerIdInvalid
+            if type(peer_id) is str:
+                peer_id = peer_id.lower().strip("@")
+
+                try:
+                    return self.peers_by_username[peer_id]
+                except KeyError:
+                    return self.resolve_username(peer_id)
+            else:
+                try:
+                    return self.peers_by_id[peer_id]
+                except KeyError:
+                    try:
+                        return self.peers_by_id[int("-100" + str(peer_id))]
+                    except KeyError:
+                        raise PeerIdInvalid
 
     def get_me(self):
         """A simple method for testing the user authorization. Requires no parameters.
@@ -471,6 +601,7 @@ class Client:
     def send_message(self,
                      chat_id: int or str,
                      text: str,
+                     parse_mode: str = "",
                      disable_web_page_preview: bool = None,
                      disable_notification: bool = None,
                      reply_to_msg_id: int = None):
@@ -485,6 +616,11 @@ class Client:
             text (:obj:`str`):
                 Text of the message to be sent.
 
+            parse_mode (:obj:`str`):
+                Use :obj:`pyrogram.ParseMode.MARKDOWN` or :obj:`pyrogram.ParseMode.HTML` if you want Telegram apps
+                to show bold, italic, fixed-width text or inline URLs in your message.
+                Defaults to Markdown.
+
             disable_web_page_preview (:obj:`bool`, optional):
                 Disables link previews for links in this message.
 
@@ -501,6 +637,8 @@ class Client:
         Raises:
             :class:`pyrogram.Error`
         """
+        style = self.html if parse_mode.lower() == "html" else self.markdown
+
         return self.send(
             functions.messages.SendMessage(
                 peer=self.resolve_peer(chat_id),
@@ -508,7 +646,7 @@ class Client:
                 silent=disable_notification or None,
                 reply_to_msg_id=reply_to_msg_id,
                 random_id=self.rnd_id(),
-                **self.markdown.parse(text)
+                **style.parse(text)
             )
         )
 
@@ -557,6 +695,7 @@ class Client:
                    chat_id: int or str,
                    photo: str,
                    caption: str = "",
+                   parse_mode: str = "",
                    ttl_seconds: int = None,
                    disable_notification: bool = None,
                    reply_to_message_id: int = None):
@@ -575,6 +714,11 @@ class Client:
             caption (:obj:`bool`, optional):
                 Photo caption, 0-200 characters.
 
+            parse_mode (:obj:`str`):
+                Use :obj:`pyrogram.ParseMode.MARKDOWN` or :obj:`pyrogram.ParseMode.HTML` if you want Telegram apps
+                to show bold, italic, fixed-width text or inline URLs in your caption.
+                Defaults to Markdown.
+
             ttl_seconds (:obj:`int`, optional):
                 Self-Destruct Timer.
                 If you set a timer, the photo will self-destruct in :obj:`ttl_seconds`
@@ -593,6 +737,7 @@ class Client:
         Raises:
             :class:`pyrogram.Error`
         """
+        style = self.html if parse_mode.lower() == "html" else self.markdown
         file = self.save_file(photo)
 
         while True:
@@ -607,7 +752,7 @@ class Client:
                         silent=disable_notification or None,
                         reply_to_msg_id=reply_to_message_id,
                         random_id=self.rnd_id(),
-                        **self.markdown.parse(caption)
+                        **style.parse(caption)
                     )
                 )
             except FilePartMissing as e:
@@ -619,6 +764,7 @@ class Client:
                    chat_id: int or str,
                    audio: str,
                    caption: str = "",
+                   parse_mode: str = "",
                    duration: int = 0,
                    performer: str = None,
                    title: str = None,
@@ -641,6 +787,11 @@ class Client:
             caption (:obj:`str`, optional):
                 Audio caption, 0-200 characters.
 
+            parse_mode (:obj:`str`):
+                Use :obj:`pyrogram.ParseMode.MARKDOWN` or :obj:`pyrogram.ParseMode.HTML` if you want Telegram apps
+                to show bold, italic, fixed-width text or inline URLs in your caption.
+                Defaults to Markdown.
+
             duration (:obj:`int`, optional):
                 Duration of the audio in seconds.
 
@@ -663,6 +814,7 @@ class Client:
         Raises:
             :class:`pyrogram.Error`
         """
+        style = self.html if parse_mode.lower() == "html" else self.markdown
         file = self.save_file(audio)
 
         while True:
@@ -685,7 +837,7 @@ class Client:
                         silent=disable_notification or None,
                         reply_to_msg_id=reply_to_message_id,
                         random_id=self.rnd_id(),
-                        **self.markdown.parse(caption)
+                        **style.parse(caption)
                     )
                 )
             except FilePartMissing as e:
@@ -697,6 +849,7 @@ class Client:
                       chat_id: int or str,
                       document: str,
                       caption: str = "",
+                      parse_mode: str = "",
                       disable_notification: bool = None,
                       reply_to_message_id: int = None):
         """Use this method to send general files.
@@ -714,6 +867,11 @@ class Client:
             caption (:obj:`str`, optional):
                 Document caption, 0-200 characters.
 
+            parse_mode (:obj:`str`):
+                Use :obj:`pyrogram.ParseMode.MARKDOWN` or :obj:`pyrogram.ParseMode.HTML` if you want Telegram apps
+                to show bold, italic, fixed-width text or inline URLs in your caption.
+                Defaults to Markdown.
+
             disable_notification (:obj:`bool`, optional):
                 Sends the message silently.
                 Users will receive a notification with no sound.
@@ -727,6 +885,7 @@ class Client:
         Raises:
             :class:`pyrogram.Error`
         """
+        style = self.html if parse_mode.lower() == "html" else self.markdown
         file = self.save_file(document)
 
         while True:
@@ -744,7 +903,7 @@ class Client:
                         silent=disable_notification or None,
                         reply_to_msg_id=reply_to_message_id,
                         random_id=self.rnd_id(),
-                        **self.markdown.parse(caption)
+                        **style.parse(caption)
                     )
                 )
             except FilePartMissing as e:
@@ -755,10 +914,11 @@ class Client:
     def send_video(self,
                    chat_id: int or str,
                    video: str,
+                   caption: str = "",
+                   parse_mode: str = "",
                    duration: int = 0,
                    width: int = 0,
                    height: int = 0,
-                   caption: str = "",
                    disable_notification: bool = None,
                    reply_to_message_id: int = None):
         """Use this method to send video files.
@@ -773,6 +933,14 @@ class Client:
                 Video to send.
                 Pass a file path as string to send a video that exists on your local machine.
 
+            caption (:obj:`str`, optional):
+                Video caption, 0-200 characters.
+
+            parse_mode (:obj:`str`):
+                Use :obj:`pyrogram.ParseMode.MARKDOWN` or :obj:`pyrogram.ParseMode.HTML` if you want Telegram apps
+                to show bold, italic, fixed-width text or inline URLs in your caption.
+                Defaults to Markdown.
+
             duration (:obj:`int`, optional):
                 Duration of sent video in seconds.
 
@@ -782,9 +950,6 @@ class Client:
             height (:obj:`int`, optional):
                 Video height.
 
-            caption (:obj:`str`, optional):
-                Video caption, 0-200 characters.
-
             disable_notification (:obj:`bool`, optional):
                 Sends the message silently.
                 Users will receive a notification with no sound.
@@ -798,6 +963,7 @@ class Client:
         Raises:
             :class:`pyrogram.Error`
         """
+        style = self.html if parse_mode.lower() == "html" else self.markdown
         file = self.save_file(video)
 
         while True:
@@ -819,7 +985,7 @@ class Client:
                         silent=disable_notification or None,
                         reply_to_msg_id=reply_to_message_id,
                         random_id=self.rnd_id(),
-                        **self.markdown.parse(caption)
+                        **style.parse(caption)
                     )
                 )
             except FilePartMissing as e:
@@ -831,6 +997,7 @@ class Client:
                    chat_id: int or str,
                    voice: str,
                    caption: str = "",
+                   parse_mode: str = "",
                    duration: int = 0,
                    disable_notification: bool = None,
                    reply_to_message_id: int = None):
@@ -849,6 +1016,11 @@ class Client:
             caption (:obj:`str`, optional):
                 Voice message caption, 0-200 characters.
 
+            parse_mode (:obj:`str`):
+                Use :obj:`pyrogram.ParseMode.MARKDOWN` or :obj:`pyrogram.ParseMode.HTML` if you want Telegram apps
+                to show bold, italic, fixed-width text or inline URLs in your caption.
+                Defaults to Markdown.
+
             duration (:obj:`int`, optional):
                 Duration of the voice message in seconds.
 
@@ -865,6 +1037,7 @@ class Client:
         Raises:
             :class:`pyrogram.Error`
         """
+        style = self.html if parse_mode.lower() == "html" else self.markdown
         file = self.save_file(voice)
 
         while True:
@@ -885,7 +1058,7 @@ class Client:
                         silent=disable_notification or None,
                         reply_to_msg_id=reply_to_message_id,
                         random_id=self.rnd_id(),
-                        **self.markdown.parse(caption)
+                        **style.parse(caption)
                     )
                 )
             except FilePartMissing as e:
@@ -1192,6 +1365,7 @@ class Client:
                           chat_id: int or str,
                           message_id: int,
                           text: str,
+                          parse_mode: str = "",
                           disable_web_page_preview: bool = None):
         """Use this method to edit text messages.
 
@@ -1207,25 +1381,33 @@ class Client:
             text (:obj:`str`):
                 New text of the message.
 
+            parse_mode (:obj:`str`):
+                Use :obj:`pyrogram.ParseMode.MARKDOWN` or :obj:`pyrogram.ParseMode.HTML` if you want Telegram apps
+                to show bold, italic, fixed-width text or inline URLs in your message.
+                Defaults to Markdown.
+
             disable_web_page_preview (:obj:`bool`, optional):
                 Disables link previews for links in this message.
 
         Raises:
             :class:`pyrogram.Error`
         """
+        style = self.html if parse_mode.lower() == "html" else self.markdown
+
         return self.send(
             functions.messages.EditMessage(
                 peer=self.resolve_peer(chat_id),
                 id=message_id,
                 no_webpage=disable_web_page_preview or None,
-                **self.markdown.parse(text)
+                **style.parse(text)
             )
         )
 
     def edit_message_caption(self,
                              chat_id: int or str,
                              message_id: int,
-                             caption: str):
+                             caption: str,
+                             parse_mode: str = ""):
         """Use this method to edit captions of messages.
 
         Args:
@@ -1240,14 +1422,21 @@ class Client:
             caption (:obj:`str`):
                 New caption of the message.
 
+            parse_mode (:obj:`str`):
+                Use :obj:`pyrogram.ParseMode.MARKDOWN` or :obj:`pyrogram.ParseMode.HTML` if you want Telegram apps
+                to show bold, italic, fixed-width text or inline URLs in your caption.
+                Defaults to Markdown.
+
         Raises:
             :class:`pyrogram.Error`
         """
+        style = self.html if parse_mode.lower() == "html" else self.markdown
+
         return self.send(
             functions.messages.EditMessage(
                 peer=self.resolve_peer(chat_id),
                 id=message_id,
-                **self.markdown.parse(caption)
+                **style.parse(caption)
             )
         )
 
